import platform
from enum import Enum

class Platform(Enum):
    LINUX = 'linux'
    MAC = 'darwin'
    WINDOWS = 'windows'

    def __bool__(self):
        return platform.system().lower() == self.value


import webbrowser
import threading
from mscan import settings
from mscan.scanner.interface import InterfaceManager, Interface
from mscan.scanner.scanner import Scanner
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from starlette.staticfiles import StaticFiles
from starlette.responses import FileResponse
import uvicorn

app = FastAPI()
scanner = Scanner()
imanager = InterfaceManager()

origins = ["*"]
app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

@app.get("/")
async def index_route():
    return FileResponse(settings.UI_BUILD_PATH / "index.html")

@app.get("/interfaces")
async def interfaces_route():
    """ Get list of interfaces """
    interfaces = [i.to_dict() for i in imanager.get_interfaces()]
    return {"interfaces": interfaces, "default": imanager.get_default()}

@app.post("/scan")
async def scan_route(interface: Interface):
    res = await scanner.scan(interface)
    return res

app.mount("/", StaticFiles(directory=settings.UI_BUILD_PATH), name="public")

def open_in_browser():
    webbrowser.open(f'{settings.PROTOCOL}://{settings.HOST}:{settings.PORT}')

def main():
    threading.Timer(1.25, open_in_browser).start()
    uvicorn.run(
        "main:app" if settings.DEV_MODE else app, 
        host=settings.HOST,
        port=settings.PORT,
        reload=settings.DEV_MODE,
        workers=1
    )
if __name__ == '__main__':
    main()



from typing import List
from icmplib import async_ping
import asyncio

async def async_multiping(addresses, count=2, interval=0.5, timeout=2,
        concurrent_tasks=50, source=None, family=None, privileged=True, progress_callback = None):

    loop = asyncio.get_running_loop()
    tasks = []
    tasks_pending = set()
    total = len(addresses)

    for i, address in enumerate(addresses):
        if len(tasks_pending) >= concurrent_tasks:
            _, tasks_pending = await asyncio.wait(
                tasks_pending,
                return_when=asyncio.FIRST_COMPLETED)
        
        if progress_callback:
            progress_callback(i, total)

        task = loop.create_task(
            async_ping(
                address=address,
                count=count,
                interval=interval,
                timeout=timeout,
                source=source,
                family=family,
                privileged=privileged,
                ))

        tasks.append(task)
        tasks_pending.add(task)

    await asyncio.wait(tasks_pending)

    return [task.result() for task in tasks]

class MultiPinger:
    """ Multithreaded class for pinging hosts and display progress """

    def progress_cb(self, finished, total):
        # print(f'total: {total} finished: {finished}')
        self._progress = finished / total * 100

    async def ping(self, ips):
        return await async_multiping(ips, count=1, timeout=0.5, progress_callback=self.progress_cb, privileged=False)
        




from dataclasses import dataclass
from typing import List
from .interface import Interface
import ipaddress

@dataclass
class Network:
    default_gateway: str
    ips: List[str]

    @classmethod
    def from_interface(cls, interface: Interface):
        network =  ipaddress.IPv4Network(f'{interface.address}/{interface.prefix_length}', strict=False)
        default_gateway = network[1]
        return cls(default_gateway, network)


from .interface import InterfaceManager, Interface
from .platform_detector import Platform
import subprocess
import re
from dataclasses import dataclass
from typing import List, Union

@dataclass
class Host:
    addr: str
    hwaddr: str
    addr_type: str

class Arp:
    
    @staticmethod
    def parse(output, iface_ip) -> List[Host]:
        lines = output.strip().split("\n")
        lines = [l.strip() for l in lines if l.strip() != ""]
        tables = {}
        current = None
        for line in lines:
            if line.startswith('Interface'):
                current = line.split()[1]
                tables[current] = []
            elif "Physical" not in line:
                addr, hwaddr, addr_type = line.split()
                row = Host(addr, hwaddr, addr_type)
                tables[current].append(row)
        return tables.get(iface_ip)

    @staticmethod
    def get_tables(interface: Interface):
        if Platform.WINDOWS:
            output = subprocess.run(["arp", "-a"], stdout=subprocess.PIPE).stdout.decode("utf-8")
            tables = Arp.parse(output, interface.address)
            return tables or []
        elif Platform.LINUX:
            hosts = []
            with open('/proc/net/arp', 'r') as f:
                lines = f.read().splitlines()
                lines = lines[1:]
            for line in lines:
                ip, hw_type, flags, hw_addr, _mask, ifname = line.split()
                print(ifname, interface.name)
                if ifname == interface.name and flags != "0x0":
                    hosts.append(Host(ip, hw_addr, hw_type))
            return hosts


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mscan', 'mscan.scanner']

package_data = \
{'': ['*'],
 'mscan': ['static/*',
           'static/static/css/*',
           'static/static/js/*',
           'static/static/media/*']}

install_requires = \
['fastapi>=0.89.1,<0.90.0',
 'icmplib>=3.0.3,<4.0.0',
 'mac-vendor-lookup>=0.1.12,<0.2.0',
 'psutil>=5.9.4,<6.0.0',
 'uvicorn[standard]>=0.20.0,<0.21.0']

entry_points = \
{'console_scripts': ['mscan = mscan.main:main']}

setup_kwargs = {
    'name': 'modern-scan',
    'version': '0.1.2',
    'description': 'Modern network scanner',
    'long_description': '# Mscan\n\n<img with="150px" height="150px" src="https://user-images.githubusercontent.com/61390950/182006521-350c306a-2567-49eb-b77a-42224783768f.png">  \n\n## A modern network scanner\n## 🖥 User Interface\n![image](https://user-images.githubusercontent.com/61390950/182006567-5e5cfcbe-7549-4205-902e-54706d2c1793.png)\n\n\n## ✨ Features\n\n- 🕹 Simple and clear user interface\n- 🚀 Super fast\n- 🛠 More features in development\n',
    'author': 'thewh1teagle',
    'author_email': '61390950+thewh1teagle@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
